/*
** Header for additional routines used by setpe/checkip
**
** Copyright (C) 2004  Moreno 'baro' Baricevic
**                                      <baro AT democritos DOT it>
**
** This  file may be used subject to the terms and conditions of the
** GNU Library General Public License as published by the Free Soft-
** ware Foundation; either version 2 of the License, or (at your op-
** tion) any later version.
**
** This file is distributed in the hope that it will be useful,  but
** WITHOUT  ANY  WARRANTY; without even the implied warranty of MER-
** CHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Li-
** brary General Public License for more details.
*/

/**************************************************
 * Author: Moreno 'baro' Baricevic                *
 * Date:   22 Oct 2004                            *
 * Title:  checkip.h                              *
 *================================================*
 * Prev Modified: 22 Oct 2004                     *
 * Last Modified: 23 Oct 2004                     *
 **************************************************/

#ifndef _CHECKIP_H_
#define _CHECKIP_H_


#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <sys/ioctl.h>		// ioctl
#include <netinet/in.h>		// sockaddr_in
#include <net/if.h>		// if_nameindex, if_freenameindex
#include <linux/sockios.h>	// SIOCGIFADDR

#include <netdb.h>		// gethostbyname
#include <sys/socket.h>		// inet_addr
#include <netinet/in.h>		// inet_addr, ntohl, htonl
#include <arpa/inet.h>		// inet_addr


#ifndef SIOCGIFADDR
# error SIOCGIFADDR unsupported by kernel!
#endif


#define new( object )	(object *)xcmalloc( sizeof( object ) )
#define XFREE( ptr )	xfree( (void *)&ptr )



/**************************************************************
** Display an IP address in readable format (include/linux/kernel.h)
*/

/* address must be in network byte order */
#define NIPQUAD(addr) \
	((unsigned char *)&addr)[0], \
	((unsigned char *)&addr)[1], \
	((unsigned char *)&addr)[2], \
	((unsigned char *)&addr)[3]

/* address must be in host byte order */
#define HIPQUAD(addr) \
	((unsigned char *)&addr)[3], \
	((unsigned char *)&addr)[2], \
	((unsigned char *)&addr)[1], \
	((unsigned char *)&addr)[0]

/* Decimal Quad Notation format */
#define QUADFMT	"%u.%u.%u.%u"



/**************************************************************
** length of the IP address string in Dotted Quad Notation
*/
#define IP_LEN	15+1



/**************************************************************
** interface structure:
**	naddr: ip address in network byte order
**	haddr: ip address in host byte order
**	iface: interface name (eth0,...)
**	ip:    ip address as string in Dotted Quad Notation
**	next:  pointer to next interface (or NULL)
*/
typedef struct _if_s  if_s;

struct _if_s {
   ulong naddr;
   ulong haddr;
   char iface [ IF_NAMESIZE ] ;
   char ip    [ IP_LEN      ] ;
   if_s * next;
};



/**************************************************************
** routines exported by checkip.c
*/
extern char * hip2str( ulong hip );
extern char * nip2str( ulong nip );
extern void clean_ifs( if_s * ptr );
extern if_s * get_myips( void );
extern ulong get_hip( const char * hostname );



/**************************************************************
** debug stuff
*/

#ifdef DEBUG
# define DBGprint( fmt , args... )				\
	do {							\
	   fprintf( stderr					\
	            , "[%-18s:%4d][%-24.24s] " fmt "\n"		\
	            , __FILE__ , __LINE__ , __FUNCTION__	\
	            , ## args					\
	          );						\
	} while ( 0 )
#else
# define DBGprint( _... )	(void)0
#endif



#endif /* _CHECKIP_H_ */



/***********************  E N D   O F   F I L E  ************************/

