/* see setpe.c for copyright info */

/**************************************************
 * Author: Moreno 'baro' Baricevic                *
 * Date:   07 Feb 2005                            *
 * Title:  ip_setpe.c                             *
 *================================================*
 * Prev Modified: 07 Feb 2005                     *
 * Last Modified: 08 Feb 2005                     *
 **************************************************/


#include "setpe.h"


typedef unsigned int uquad_t[4];


/************************************************************************
 *_____________________________  CHECK_CNT _____________________________*
 ************************************************************************/
static void check_cnt( int n )
{
   if ( mosnet[n].cnt < 0 )
   {
      fprintf( stderr , "%s: Negative count\n" , pname );
      exit( ERR_OTHER );
   }
   if ( n > MAX_MOSNET_ENTS )
   {
      fprintf( stderr , "%s: Table Overflow\n" , pname );
      exit( ERR_OTHER );
   }
} /* check_cnt */



/**********************************************************************
 *_____________________________  IP_LINE _____________________________*
 **********************************************************************/
static void ip_line( int * n , uquad_t q )
{
   NET_TO_IP( mosnet[(*n)++] ) = (q[0]<<24) | (q[1]<<16) | (q[2]<<8) | q[3];
   check_cnt( *n );
} /* ip_line */



/************************************************************************
 *_____________________________  HOST_LINE _____________________________*
 ************************************************************************/
static void host_line( int * n , const char * host )
{
   u_long ipaddr = get_hip( host );
   if ( ipaddr == -1 )
     exit( ERR_OTHER );
   NET_TO_IP( mosnet[(*n)++] ) = ipaddr;
   check_cnt( *n );
} /* host_line */



/****************************************************************************
 *_____________________________  IP_READ_TABLE _____________________________*
 ****************************************************************************/
static int ip_read_table( FILE * table )
{
   char buf[513] , host[512] , word[10];
   int line = 0;
   int n = 0;
   uquad_t q;

   while( fgets( buf , 512 , table ) )
   {
      ++line;

      if ( buf[0] && buf[0] != '#' && buf[0] != '\n' )
      {
         buf[512] = '\0';

         if ( 6 == sscanf( buf , "%d %u.%u.%u.%u %d"
                               , &mosnet[n].base
                               , &q[0] , &q[1] , &q[2] , &q[3]
                               , &mosnet[n].cnt ) )
         {
            ip_line( &n , q );
            continue;
         }

         if ( 3 == sscanf( buf , "%d %s %d"
                               , &mosnet[n].base
                               , host
                               , &mosnet[n].cnt ) )
         {
            host_line( &n , host );
            continue;
         }

         if ( 6 == sscanf( buf , "%d %u.%u.%u.%u %10s"
                               , &mosnet[n].base
                               , &q[0] , &q[1] , &q[2] , &q[3]
                               , word )
              && ! strcasecmp( word , "alias" ) )
         {
            mosnet[n].cnt = 0;
            ip_line( &n , q );
            continue;
         }

         if ( 3 == sscanf( buf , "%d %s %10s"
                               , &mosnet[n].base
                               , host
                               , word )
              && ! strcasecmp( word , "alias" ) )
         {
            mosnet[n].cnt = 0;
            host_line( &n , host );
            continue;
         }

//         else
         {
            if ( buf[strlen(buf)-1] == '\n' )
              buf[strlen(buf)-1] = '\0';

            fprintf( stderr , "%s: Bad Table Input at line %d: <%s>\n"
                            , pname , line , buf );

            exit( ERR_OTHER );
         }

      } // buf
   } // while

   return n;

} /* ip_read_table */



/*******************************************************************************
 *_____________________________  IP_CHECK_ADDRESS _____________________________*
 *******************************************************************************/
static int ip_check_address( int n , u_long * myaddr )
{
   int i = 0;
   int match = 0;
   if_s * ips , * walk ;

   ips = walk = get_myips();

   if ( ! ips || ! ips->haddr )
     return -1;

   while ( walk && walk->haddr && ! match )
   {
      *myaddr = walk->haddr;

      if ( check )
        fprintf( stderr , "checking %s, ip: %s\n"
                        , walk->iface , walk->ip );

      for ( i = 0 ; i < n ; i++ )
      {
         DBGprint( "-> " QUADFMT " vs " QUADFMT "\n"
                   , HIPQUAD( *myaddr )
                   , HIPQUAD( NET_TO_IP( mosnet[i] ) ) );
         if ( *myaddr >= NET_TO_IP( mosnet[i] ) &&
              *myaddr < NET_TO_IP( mosnet[i] ) + mosnet[i].cnt )
         {
            if ( check )
              fprintf( stderr , "match found: %s/" QUADFMT "-%d\n"
                              , walk->ip
                              , HIPQUAD( NET_TO_IP( mosnet[i] ) )
                              , mosnet[i].cnt );
            match = 1 ;
            break;
         }
      }
      walk = walk->next;
   }

   if ( i == n )
   {
      fprintf( stderr , "%s: the supplied table is well-formatted,\n" , pname );
      fprintf( stderr , "but my IP address ( " );
      for ( walk = ips; walk ; walk = walk->next )
        fprintf( stderr , "%s " , walk->ip );
      fprintf( stderr , ") is not there!\n" );

      exit( ERR_OTHER );
   }

   clean_ifs( ips );

   return i;

} /* ip_check_address */



/********************************************************************************
 *_____________________________  IP_CHECK_HOSTNAME _____________________________*
 ********************************************************************************/
static int ip_check_hostname( int n , u_long * myaddr )
{
   int i = 0;
   char hostname[MAXHOSTNAMELEN + 1] = { '\0' };

   fprintf( stderr , "%s: warning: iface check failed (network not "
                     "configured?), fall back to hostname check\n"
                   , pname );

   if ( gethostname( hostname , MAXHOSTNAMELEN ) < 0 )
   {
      perror( "setpe: gethostname" );
      exit( ERR_OTHER );
   }

   *myaddr = get_hip( hostname );
   if ( *myaddr == -1 )
     exit( ERR_OTHER );

   for ( i = 0 ; i < n ; i++ )
   {
      if ( *myaddr >= NET_TO_IP( mosnet[i] ) &&
           *myaddr < NET_TO_IP( mosnet[i] ) + mosnet[i].cnt )
         break;
   }

   if ( i == n )
   {
      fprintf( stderr , "%s: the supplied table is well-formatted,\n"
                        "but my IP address (" QUADFMT ") is not there!\n"
                      , pname , HIPQUAD( *myaddr ) );
      exit( ERR_OTHER );
   }

   return i;

} /* ip_check_hostname */




/***********************************************************************
 *_____________________________  IP_SETPE _____________________________*
 ***********************************************************************/
void ip_setpe( int pe , FILE * table )
{
   int err, n = 0;
   unsigned int i;
   struct mosixnet oldmosnet[MAX_MOSNET_ENTS];
   int oldpe, oldready_pe, oldnents;

   n = ip_read_table( table );
   if ( ! n )
   {
      fprintf( stderr , "%s: the table is empty.\n" , pname );
      exit( ERR_OTHER );
   }

   if ( ! pe )
   {
      u_long myaddr;	// host byte order
      int el = ip_check_address( n , &myaddr );
      if ( el == -1 )
      {
         // network is not configured
         el = ip_check_hostname( n , &myaddr );
      }
      pe = mosnet[el].base + myaddr - NET_TO_IP( mosnet[el] );
   }

   ip_check_table( n , pe );

#ifdef TEST_CHECKIP
printf( "out\n" );
exit( 0 );
#endif

   /* set network family - should be in the first entry */
   mosnet[0].saddr.sa_family = AF_INET;

   /* lastly, the kernel expects network order for the IP's */
   for ( i = 0 ; i < n ; i++ )
     NET_TO_IP( mosnet[i] ) = htonl( NET_TO_IP( mosnet[i] ) );

   err = table_read( &oldpe , &oldready_pe , &oldnents , oldmosnet );
   if ( err )
   {
      if ( err != ENOSYS )
      {
         perror( "" );
         exit( ERR_OTHER );
      }
      /* else: silently ignore... we not configured anyway */
   }

   if ( ! err )
   {
      if ( ip_need_two_phase( pe , n , oldpe , oldmosnet , oldnents ) )
      {
         if ( ! two_phase )
         {
            fprintf( stderr , "%s: The proposed change will modify the current node's mapping,\n" , pname );
            fprintf( stderr , "requiring a full openMosix restart, including bringing all processes back home:\n" );
            fprintf( stderr , "If you are sure about the change, use 'setpe -W' (instead of 'setpe -w').\n" );
            exit( ERR_OTHER );
         }
      }
   }
   else
   {
      two_phase = 0;
   }

   if ( check )
     return;

   /*  -- COMMIT POINT --  */

   close( proc_mosix_admin_mospe_fd );
   close( proc_mosix_admin_config_fd );
   proc_mosix_admin_mospe_fd  = open( MOSPE_FILENAME  , O_WRONLY );
   proc_mosix_admin_config_fd = open( CONFIG_FILENAME , O_WRONLY );

   if ( two_phase )
   {

      /* ok then, so - shutdown and try new configuration,
         and if failed - restore old one */

      setpe_off();

      close( proc_mosix_admin_mospe_fd );
      proc_mosix_admin_mospe_fd = open( MOSPE_FILENAME , O_WRONLY );
   }

   if ( write_file( pe , mosnet , n ) == -1 && two_phase )   /* FAILED */
   {
      close( proc_mosix_admin_mospe_fd );
      close( proc_mosix_admin_config_fd );
      proc_mosix_admin_mospe_fd  = open( MOSPE_FILENAME  , O_WRONLY );
      proc_mosix_admin_config_fd = open( CONFIG_FILENAME , O_WRONLY );
      (void)write_file( oldpe , oldmosnet , oldnents );
      fprintf( stderr , "%s: configuration restored!\n" , pname );
   }

} /* ip_setpe */



/***********************  E N D   O F   F I L E  ************************/

